
#ifndef BASEOBJECT_H_INCLUDED
#define BASEOBJECT_H_INCLUDED

#include <list>
#include <point3.h>
#include <vector3.h>
#include <material.h>
#include <matrix4x4.h>
#include <vertexlist.h>

class Primitive;
typedef list<Primitive *> PrimitiveList;

class BaseObject
{
public:
    BaseObject( void ) {}
    virtual ~BaseObject( void ) {}

    virtual Matrix4x4 getTransform( void ) const { return identityMatrix(); }

    virtual PrimitiveList foldModel( const Matrix4x4 &xform ) const = 0;
    virtual VertexList foldToVerticies( const Matrix4x4 &xform ) const = 0;

    virtual void setMaterial( Material *m, bool override = true ) = 0;
    virtual void setSampleRate( int newrate ) = 0;

    /* Return intersect as a t value from the given direction and a normal. */
    virtual bool closestIntersect( const Point3 &origin, const Vector3 &dir,
                                   double *t, Vector3 *norm,
                                   Material **mat ) const = 0;
};

#endif // BASEOBJECT_H_INCLUDED
