
#include <math.h>
#include <colour.h>

static const double display_gamma = 2.2;

/**
 * Returns a nonlinear R'G'B' channel value in 0-1 from a linear RGB
 * channel value in 0-1.
 */
double linear_to_nonlinear_rgb( double val )
{
    if( val <= 0.018 ) {
        return 4.5 * val;
    }
    return ( ( 1.099 * pow( val, 1.0 / display_gamma ) ) - 0.099 );
}

/**
 * Returns a linear RGB channel value in 0-1 from a nonlinear R'G'B'
 * channel value in 0-1.
 */
double nonlinear_to_linear_rgb( double val )
{
    if( val <= 0.081 ) {
        return val / 4.5;
    }
    return pow( ( val + 0.099 ) / 1.099, display_gamma );
}

void xyz_to_rgb( double  x, double  y, double  z,
                 double *r, double *g, double *b )
{
    *r = x; *g = y; *b = z;
/*
    *r = (  3.240479 * x ) + ( -1.537150 * y ) + ( -0.498535 * z );
    *g = ( -0.969256 * x ) + (  1.875992 * y ) + (  0.041556 * z );
    *b = (  0.055648 * x ) + ( -0.204043 * y ) + (  1.057311 * z );
*/
}

void rgb_to_xyz( double  r, double  g, double  b,
                 double *x, double *y, double *z )
{
    *x = r; *y = g; *z = b;
/*
    *x = ( 0.412453 * r ) + ( 0.357580 * g ) + ( 0.180423 * b );
    *y = ( 0.212671 * r ) + ( 0.715160 * g ) + ( 0.072169 * b );
    *z = ( 0.019334 * r ) + ( 0.119193 * g ) + ( 0.950227 * b );
*/
}

void rgb_to_lab( double r, double g, double b,
                 double *lstar, double *astar, double *bstar )
{
    double x, y, z;
    rgb_to_xyz( r, g, b, &x, &y, &z );

    *lstar = pow( y, 1.0 / 3.0 );
    *astar = pow( x, 1.0 / 3.0 );
    *bstar = pow( z, 1.0 / 3.0 );
}

void lab_to_rgb( double lstar, double astar, double bstar,
                 double *r, double *g, double *b )
{
    double x, y, z;

    y = lstar * lstar * lstar;
    x = astar * astar * astar;
    z = bstar * bstar * bstar;

    xyz_to_rgb( x, y, z, r, g, b );
}

