
#include <math.h>
#include <stdio.h>
#include "pngoutput.h"
#include "cooktorrence.h"

class CTFactor1 : public TextureData
{
public:
    CTFactor1( double specr, double specg, double specb,
               double ks, double m, unsigned int s )
    {
        unsigned char *buf;
        unsigned int i, j;

        size = s;
        pixels = new unsigned char[ size * size * 4 ];
        buf = pixels;
        clamp = false;

        /**
         * Takes parameters ( v.h, n.h ).
         *                          ^----- between 0-1
         *                     ^---------- between 0-1
         */
        pngoutput_t *pngoutput = pngoutput_new( "ct-factor1.png", size, size, 0.45 );
        for( i = 0; i < size; i++ ) {
            for( j = 0; j < size; j++ ) {
                double vh = (double) j / (double) ( size - 1 ); /* x */
                double nh = (double) i / (double) ( size - 1 ); /* y */
                vh = ( vh * 2.0 ) - 1.0;
                nh = ( nh * 2.0 ) - 1.0;

                // if( nh > 0.0 ) nh -= 1.0; else nh += 1.0;

                double dis = exp( -1.0 * ((1.0 - (nh * nh))/(nh*nh*m*m)) );
                       dis = dis / (m*m*nh*nh*nh*nh);

                /* Schlick!!  Aw yea!! */
                double fresr = specr + ( (1.0 - specr) * pow( (1.0 - vh), 5.0 ) );
                double fresg = specg + ( (1.0 - specg) * pow( (1.0 - vh), 5.0 ) );
                double fresb = specb + ( (1.0 - specb) * pow( (1.0 - vh), 5.0 ) );

                double resultr = fresr * dis;
                double resultg = fresg * dis;
                double resultb = fresb * dis;

                /* Clip at 1 for fun. */
                if( resultr > 1.0 ) resultr = 1.0;
                if( resultg > 1.0 ) resultg = 1.0;
                if( resultb > 1.0 ) resultb = 1.0;

                *buf++ = (unsigned char) ( ( resultr * 255.0 ) + 0.5 );
                *buf++ = (unsigned char) ( ( resultg * 255.0 ) + 0.5 );
                *buf++ = (unsigned char) ( ( resultb * 255.0 ) + 0.5 );
                *buf++ = 0xff;
            }
            pngoutput_scanline( pngoutput, pixels + (i * size * 4) );
        }
        pngoutput_delete( pngoutput );
        success = setupTexture( pixels );
    }

    ~CTFactor1( void )
    {
        delete [] pixels;
    }

private:
    unsigned char *pixels;
};

#if 0
static inline double geometric( double v, double m )
{
/*
    double h = (v * v) / (2.0 * m * m * (1.0 - (v * v)));
    double g = sqrt( h * M_PI ) * (2.0 - erfc( sqrt( h ) ));
    return g / (g + 1.0);
*/
    double k = sqrt( (2.0 * m * m) / M_PI );
    return v / (v - (k*v) + k);
}
#endif

static inline double thingy( double cott, double m )
{
    return 0.5 * ( sqrt( 2.0 / M_PI ) * ( m / cott )
               * exp( ( -1.0 * ( ( cott * cott ) / ( 2.0 * m * m ) ) )
                      - erfc( cott / ( sqrt( 2.0 ) * m ) ) ) );
}

static inline double geometric( double theta, double m )
{
    double cott = 1.0 / tan( theta );
    double result = ( 1.0 - ( 0.5 * erfc( cott / ( m * sqrt( 2.0 ) ) ) ) );
    result = result / ( thingy( cott, m ) + 1.0 );
    return result;
}

class CTFactor2 : public TextureData
{
public:
    CTFactor2( double ks, double m, unsigned int s )
    {
        unsigned char *buf;
        unsigned int i, j;

        size = s;
        pixels = new unsigned char[ size * size * 4 ];
        buf = pixels;
        clamp = true;

        /**
         * Takes parameters ( n.l, n.v ).
         *                          ^----- between [0,1]
         *                     ^---------- between [0,1]
         */

        pngoutput_t *pngoutput = pngoutput_new( "ct-factor2.png", size, size, 0.45 );
        for( i = 0; i < size; i++ ) {
            for( j = 0; j < size; j++ ) {
                double nl = (double) j / (double) ( size - 1 ); /* x */
                double nv = (double) i / (double) ( size - 1 ); /* y */
                nl = ( nl * 2.0 ) - 1.0;
                nv = ( nv * 2.0 ) - 1.0;

                double geo = 0.0;
                if( nl >= 0.0 && nv >= 0.0 ) {
                    geo = geometric( acos( nl ), m ) * geometric( acos( nv ), m );
                }

                //double geo = 0.5;//geometric( nl, m ) * geometric( nv, m );
                //if( geo < 0.0 ) geo = 1.0;
                //if( geo > 1.0 ) geo = 1.0;
                //assert( geo >= 0.0 );
                //assert( geo <= 1.0 );

/*
                double angle_to_view = acos( nv );
                double angle_to_light = acos( nl );
                double angle_to_half = ( angle_to_view + angle_to_light ) / 2.0;
                double angle_n_h = fabs( angle_to_half - angle_to_view );
                double val1 = ( 2.0 * angle_n_h * angle_to_view ) / angle_to_half;
                double val2 = ( 2.0 * angle_n_h * angle_to_light ) / angle_to_half;
                double minval = ( 1.0 <   val1) ?  1.0 :   val1;
                       minval = (val2 < minval) ? val2 : minval;
*/

                //double result = ks * ( geo / ( M_PI * ( 0.0001 + nv ) ) );
                double result = geo;// / (0.0001+(nv*nl));// / (0.0001 + (nv*nl));// * ( geo / ( M_PI * nv ) );

                /* Clip at 1 for fun. */
                if( result > 1.0 ) result =  1.0;

                *buf++ = (unsigned char) ( ( result * 255.0 ) + 0.5 );
                *buf++ = (unsigned char) ( ( result * 255.0 ) + 0.5 );
                *buf++ = (unsigned char) ( ( result * 255.0 ) + 0.5 );
                *buf++ = 0xff;
            }
            pngoutput_scanline( pngoutput, pixels + (i * size * 4) );
        }
        pngoutput_delete( pngoutput );
        success = setupTexture( pixels );
    }

    ~CTFactor2( void )
    {
        delete [] pixels;
    }

private:
    unsigned char *pixels;
};

CookTorrence::CookTorrence( double diffr, double diffg, double diffb,
                            double specr, double specg, double specb,
                            double kd, double ks, double m, unsigned int size )
{
    f1 = new CTFactor1( specr, specg, specb, ks, m, size );
    f2 = new CTFactor2( ks, m, size );
}

CookTorrence::~CookTorrence( void )
{
    delete f1;
    delete f2;
}

void CookTorrence::rebuild( double diffr, double diffg, double diffb,
                            double specr, double specg, double specb,
                            double kd, double ks, double m, unsigned int size )
{
    delete f1;
    delete f2;
    f1 = new CTFactor1( specr, specg, specb, ks, m, size );
    f2 = new CTFactor2( ks, m, size );
}

