
#include <stdio.h>
#include <stdlib.h>
#include <GL/glut.h>
#include "vector3.h"
#include "model.h"
#include "sceneparser.h"
#include "pnginput.h"
#include "pngtexture.h"
#include "cooktorrence.h"

static Model *m;
static Vector3 camera_position( 0.0, 0.0, -2.0 );
static Vector3 camera_up( 0.0, 1.0, 0.0 );
static Vector3 camera_direction = Vector3( 0.0, 0.0, 3.6 ).normalize();
static Vector3 light_position( 0.0, 0.0, -3.6 );
static Vector3 light_up( 0.0, 1.0, 0.0 );

static TextureData *texdata;
static TextureData *lighttex;
static TextureData *mask;
static TextureData *lightmask;
static CookTorrence *ct;

double dr, dg, db;
static double ctm = 0.2;

static void init( void )
{
    glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
    glEnable( GL_LIGHTING );
    glEnable( GL_COLOR_MATERIAL );
    glEnable( GL_ALPHA_TEST );
    glEnable( GL_DEPTH_TEST );
    glEnable( GL_BLEND );
    glEnable( GL_CULL_FACE );

    glDepthFunc( GL_LEQUAL );
    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
    assert( glGetError() == GL_NO_ERROR );

    GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
    glLightfv( GL_LIGHT0, GL_DIFFUSE, (const GLfloat *) lightDiffuse );
}

static void display( void )
{
    /* Clear the colour and depth. */
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    /* Setup camera position. */
    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
    gluLookAt( camera_position.getX(),
               camera_position.getY(),
               camera_position.getZ(),
               camera_position.getX() + camera_direction.getX(),
               camera_position.getY() + camera_direction.getY(),
               camera_position.getZ() + camera_direction.getZ(),
               camera_up.getX(), camera_up.getY(), camera_up.getZ() );

    /* Setup our light position. */
    GLfloat lightPosition[4];
    lightPosition[ 0 ] = light_position.getX();
    lightPosition[ 1 ] = light_position.getY();
    lightPosition[ 2 ] = light_position.getZ();
    lightPosition[ 3 ] = 1.0;
    glLightfv( GL_LIGHT0, GL_POSITION, (const GLfloat *) lightPosition );
    glEnable( GL_LIGHT0 );


    glEnable( GL_TEXTURE_2D );

/*
    glActiveTextureARB( GL_TEXTURE0_ARB );
    texdata->bindTexture();
    glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

    glActiveTextureARB( GL_TEXTURE1_ARB );
    mask->bindTexture();
    glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

    glEnable( GL_DEPTH_TEST );
    glDisable( GL_BLEND );
    glEnable( GL_LIGHTING );
    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
    glColor4f( 1.0, 1.0, 1.0, 1.0 );

    m->drawModelWithMask();


    glActiveTextureARB( GL_TEXTURE0_ARB );
    lighttex->bindTexture();
    glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

    glActiveTextureARB( GL_TEXTURE1_ARB );
    lightmask->bindTexture();
    glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

    glBlendFunc( GL_SRC_ALPHA, GL_ONE );
    glEnable( GL_BLEND );
    glColor4f( 1.0, 1.0, 1.0, 1.0 );
    m->drawModelWithLightingAndMask( Point3::origin.add( camera_position ),
                                     Point3::origin.add( light_position ) );

*/


    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
    glColor4f( 1.0, 1.0, 1.0, 1.0 );
    ct->getTexture1()->bindTexture();
    m->drawModelWithLighting2_1( Point3::origin.add( camera_position ),
                                 Point3::origin.add( light_position ) );

    glBlendFunc( GL_DST_COLOR, GL_ZERO );
    glEnable( GL_BLEND );
    ct->getTexture2()->bindTexture();
    m->drawModelWithLighting2_2( Point3::origin.add( camera_position ),
                                 Point3::origin.add( light_position ) );

    //glBlendFunc( GL_SRC_ALPHA, GL_ONE );
    //glEnable( GL_BLEND );
    //glColor4f( 0.9, 0.5, 0.3, 1.0 );
    //lighttex->bindTexture();
    //m->drawModelWithLighting( Point3::origin.add( camera_position ),
    //                          Point3::origin.add( light_position ) );

    glDisable( GL_TEXTURE_2D );
    glColor4f( dr, dg, db, 0.5 );
    glBlendFunc( GL_SRC_ALPHA, GL_ONE );
    glEnable( GL_BLEND );
    m->drawModel();

    glFlush();
    glutSwapBuffers();
}

static void reshape( int w, int h )
{
    glViewport( 0, 0, (GLsizei) w, (GLsizei) h );
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();
    gluPerspective( 30.0, (GLfloat) w / (GLfloat) h, 0.01, 30.0 );
    glMatrixMode( GL_MODELVIEW );
}

static void keyboard( unsigned char key, int x, int y )
{
    switch( key ) {
        case 27: exit( 0 ); break;
        case 'q': light_position = light_position.add( Vector3( 0.5, 0.0, 0.0 ) );
                  glutPostRedisplay();
                  break;
        case 'a': light_position = light_position.add( Vector3( -0.5, 0.0, 0.0 ) );
                  glutPostRedisplay();
                  break;
        case 'w': light_position = light_position.add( Vector3( 0.0, 0.5, 0.0 ) );
                  glutPostRedisplay();
                  break;
        case 's': light_position = light_position.add( Vector3( 0.0, -0.5, 0.0 ) );
                  glutPostRedisplay();
                  break;
        case 'e': light_position = light_position.add( Vector3( 0.0, 0.0, 0.5 ) );
                  glutPostRedisplay();
                  break;
        case 'd': light_position = light_position.add( Vector3( 0.0, 0.0, -0.5 ) );
                  glutPostRedisplay();
                  break;

        case 'r':
            camera_position = camera_position.rotate( 0.1, Vector3( 0.0, 1.0, 0.0 ) );
            camera_direction = camera_position.scalarMult( -1.0 ).normalize();
            glutPostRedisplay();
            break;
        case 'f':
            camera_position = camera_position.rotate( -0.1, Vector3( 0.0, 1.0, 0.0 ) );
            camera_direction = camera_position.scalarMult( -1.0 ).normalize();
            glutPostRedisplay();
            break;
        case 'o':
            ctm += 0.1;
            if( ctm > 1.0 ) ctm = 1.0;
            ct->rebuild( 0.75164f,  0.60648f,  0.22648f,
                         0.728281f, 0.655802f, 0.166065f,
                         0.5, 0.5, ctm, 256 );
            glutPostRedisplay();
            break;
        case 'l':
            ctm -= 0.1;
            if( ctm < 0.1 ) ctm = 0.1;
            ct->rebuild( 0.75164f,  0.60648f,  0.22648f,
                         0.728281f, 0.655802f, 0.166065f,
                         0.5, 0.5, ctm, 256 );
            glutPostRedisplay();
            break;

        default: fprintf( stderr, "normal: %d\n", key ); break;
    }
}

static void special_keyboard( int key, int x, int y )
{
    switch( key ) {
        case 101: /* Up. */
            camera_position = camera_position.add( camera_direction.scalarMult( 0.1 ) );
            glutPostRedisplay();
            break;
        case 103: /* Down. */
            camera_position = camera_position.add( camera_direction.scalarMult( -0.1 ) );
            glutPostRedisplay();
            break;
        case 100: /* Left. */
            camera_direction = camera_direction.rotate( 0.1, Vector3( 0.0, 1.0, 0.0 ) ).normalize();
            glutPostRedisplay();
            break;
        case 102: /* Right. */
            camera_direction = camera_direction.rotate( -0.1, Vector3( 0.0, 1.0, 0.0 ) ).normalize();
            glutPostRedisplay();
            break;
        default: fprintf( stderr, "special: %d\n", key ); break;
    }
}

static int last_x, last_y;
static bool move_camera = true;
static bool zoom_camera = false;

static void mouse_motion( int x, int y )
{
    if( move_camera ) {
        if( zoom_camera ) {
            camera_position = camera_position.rotate( (last_x - x) * 0.01,
                                                      camera_up );
            camera_direction = camera_position.scalarMult( -1.0 ).normalize();
            camera_position = camera_position.add( camera_direction.scalarMult( (last_y - y) * 0.01 ) );
        } else {
            Vector3 perp = camera_position.normalize().cross( camera_up.normalize() );

            camera_up = camera_up.rotate( (y - last_y) * 0.01, perp );
            camera_position = camera_position.rotate( (y - last_y) * 0.01, perp );
            camera_position = camera_position.rotate( (last_x - x) * 0.01,
                                                      camera_up );
            camera_direction = camera_position.scalarMult( -1.0 ).normalize();
        }
    } else {
        Vector3 perp = light_position.normalize().cross( light_up.normalize() );

        light_position = light_position.rotate( (x - last_x) * 0.01,
                                                light_up );
        light_up = light_up.rotate( (last_y - y) * 0.01, perp );
        light_position = light_position.rotate( (last_y - y) * 0.01, perp );
    }

    last_x = x;
    last_y = y;
    glutPostRedisplay();
}

static void mouse_button( int button, int state, int x, int y )
{
    last_x = x;
    last_y = y;
}

static void menu_func( int value )
{
    move_camera = ( value == 1 || value == 2 );
    zoom_camera = ( value == 2 );
}

int main( int argc, char **argv )
{
    SceneParser p( argv[ 1 ] );
    Node *teapot = p.getNode( "teapot" );
    m = new Model( teapot->foldToVerticies( identityMatrix() ) );
    m->centreModel();

    glutInit( &argc, argv );
    glutInitDisplayMode( GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE );
    glutInitWindowSize( 512, 512 );
    glutInitWindowPosition( 100, 100 );
    glutCreateWindow( argv[ 0 ] );
    init();

    glutMotionFunc( mouse_motion );
    glutMouseFunc( mouse_button );

    glutCreateMenu( menu_func );
    glutAddMenuEntry( "Move Camera", 1 );
    glutAddMenuEntry( "Zoom Camera", 2 );
    glutAddMenuEntry( "Move Light", 3 );
    glutAttachMenu( GLUT_RIGHT_BUTTON );

    glutDisplayFunc( display );
    glutReshapeFunc( reshape );
    glutKeyboardFunc( keyboard );
    glutSpecialFunc( special_keyboard );

    // Load textures.
    texdata = new PNGTexture( argv[ 2 ] );
    mask = new PNGTexture( argv[ 3 ] );
    //lighttex = new ATex( 512.0, 1024 );
    // atex2 = new ATex2( 1000.0, 3.0, 1024 );
    lightmask = new PNGTexture( argv[ 4 ] );
    //ct = new CookTorrence( 0.35, 1024 );

    ct = new CookTorrence( 0.0112648f, 0.725164f,  0.990648f,
                           0.106065f, 0.525164f, 0.955802f,
                           0.5, 0.1, 0.22, 1024 );
    dr = 0.0112648f; dg = 0.725164f; db = 0.990648f;

    /*
    ct = new CookTorrence( 0.75164f,  0.60648f,  0.22648f,
                           0.728281f, 0.655802f, 0.166065f,
                           0.5, 0.5, ctm, 256 );
    dr = 0.75164f; dg = 0.60648f; db = 0.22648f;
    */

    if( !texdata->wasSuccessful() || !lightmask->wasSuccessful() ) return 0;

    glutMainLoop();
    return 0;
}

