
#ifndef MATERIAL_H_INCLUDED
#define MATERIAL_H_INCLUDED

#include <string>
#include <pixelxyz.h>

class Material
{
public:
    Material( void )
        : nu( 0.0 ), nv( 0.0 ), ks( 0.0 ), kd( 1.0 ), l( false ) {}
    ~Material( void ) {}

    void setDiffuseColour( const PixelXYZ &newc ) { c = newc; }
    void setSpecularColour( const PixelXYZ &newc ) { sc = newc; }
    void setLuminous( const bool &newl ) { l = newl; }
    void setName( const char *name ) { matname = name; }
    void setNu( const double &newnu ) { nu = newnu; }
    void setNv( const double &newnv ) { nv = newnv; }
    void setKs( const double &newks ) { ks = newks; }
    void setKd( const double &newkd ) { kd = newkd; }

    const char *getName( void ) const { return matname.c_str(); }
    PixelXYZ getDiffuseColour( void ) const { return c; }
    PixelXYZ getSpecularColour( void ) const { return sc; }
    bool isLuminous( void ) const { return l; }
    double getNu( void ) const { return nu; }
    double getNv( void ) const { return nv; }
    double getKs( void ) const { return ks; }
    double getKd( void ) const { return kd; }

private:
    std::string matname;
    PixelXYZ c;
    PixelXYZ sc;
    double nu, nv, ks, kd;
    bool l;
};

#endif // MATERIAL_H_INCLUDED
