
#include <stdio.h>
#include <GL/gl.h>
#include "model.h"

Model::Model( const VertexList &vertices )
    : vlist( vertices )
{
}

Model::~Model( void )
{
}

void Model::centreModel( void )
{
    VertexList::const_iterator vit;
    VertexList newlist;
    int count = 0;
    double totalx = 0.0, totaly = 0.0, totalz = 0.0;

    for( vit = vlist.begin(); vit != vlist.end(); vit++ ) {
        totalx += (*vit).pos.getX();
        totaly += (*vit).pos.getY();
        totalz += (*vit).pos.getZ();
        count++;
    }
    totalx = totalx / (double) count;
    totaly = totaly / (double) count;
    totalz = totalz / (double) count;
    totalx = -totalx;
    totaly = -totaly;
    totalz = -totalz;

    for( vit = vlist.begin(); vit != vlist.end(); vit++ ) {
        Vertex curvert;
        curvert.pos = (*vit).pos.add( Vector3( totalx, totaly, totalz ) );
        curvert.normal = (*vit).normal;
        curvert.hasnormal = (*vit).hasnormal;
        newlist.push_back( curvert );
    }

    vlist = newlist;
}

void Model::drawModel( void )
{
    VertexList::const_iterator vit;

    glBegin( GL_TRIANGLES );
    for( vit = vlist.begin(); vit != vlist.end(); vit++ ) {
        double val1 = ( asin( (*vit).normal.getX() ) / M_PI ) + 0.5;
        double val2 = ( asin( (*vit).normal.getY() ) / M_PI ) + 0.5;

        glMultiTexCoord2fARB( GL_TEXTURE0_ARB, val1, val2 );
        glMultiTexCoord2fARB( GL_TEXTURE1_ARB, val1, val2 );
        glNormal3f( (*vit).normal.getX(),
                    (*vit).normal.getY(),
                    (*vit).normal.getZ() );
        glVertex3f( (*vit).pos.getX(),
                    (*vit).pos.getY(),
                    (*vit).pos.getZ() );
    }
    glEnd();
}

void Model::drawModelWithLighting_vh_nh( const Point3 &eyepos, const Point3 &lpos )
{
    VertexList::const_iterator vit;

    glBegin( GL_TRIANGLES );
    for( vit = vlist.begin(); vit != vlist.end(); vit++ ) {
        Vector3 v = eyepos.subtract( (*vit).pos ).normalize();
        Vector3 l = lpos.subtract( (*vit).pos ).normalize();
        Vector3 h = v.add( l ).normalize();
        Vector3 n = (*vit).normal;

        double vh = v.dotProduct( h );
        double nh = n.dotProduct( h );
        double val1 = ( asin( (*vit).normal.getX() ) / M_PI ) + 0.5;
        double val2 = ( asin( (*vit).normal.getY() ) / M_PI ) + 0.5;

        assert( !isnan( vh ) && !isinf( vh ) );
        assert( !isnan( nh ) && !isinf( nh ) );

        vh = ( vh / 2.0 ) + 0.5;
        nh = ( nh / 2.0 ) + 0.5;
        //nh = 1.0 - nh;
        //vh = 1.0 - vh;

        glTexCoord2f( vh, nh );
        glMultiTexCoord2fARB( GL_TEXTURE1_ARB, val1, val2 );
        glNormal3f( (*vit).normal.getX(),
                    (*vit).normal.getY(),
                    (*vit).normal.getZ() );
        glVertex3f( (*vit).pos.getX(),
                    (*vit).pos.getY(),
                    (*vit).pos.getZ() );
    }
    glEnd();
}

void Model::drawModelWithLighting_nl_nv( const Point3 &eyepos, const Point3 &lpos )
{
    VertexList::const_iterator vit;

    glBegin( GL_TRIANGLES );
    for( vit = vlist.begin(); vit != vlist.end(); vit++ ) {
        Vector3 v = eyepos.subtract( (*vit).pos ).normalize();
        Vector3 l = lpos.subtract( (*vit).pos ).normalize();
        Vector3 n = (*vit).normal;

        double nl = n.dotProduct( l );
        double nv = n.dotProduct( v );
        double val1 = ( asin( (*vit).normal.getX() ) / M_PI ) + 0.5;
        double val2 = ( asin( (*vit).normal.getY() ) / M_PI ) + 0.5;

        assert( !isnan( nl ) && !isinf( nl ) );
        assert( !isnan( nv ) && !isinf( nv ) );
        assert( nl >= -1.0 && nl <= 1.0 );
        assert( nv >= -1.0 && nv <= 1.0 );

        nl = ( nl / 2.0 ) + 0.5;
        nv = ( nv / 2.0 ) + 0.5;
        //nv = 1.0 - nv;

        if( nl > 0.9 || nv > 0.9 ) nl = nv = 0.9;

        glTexCoord2f( nl, nv );
        glMultiTexCoord2fARB( GL_TEXTURE1_ARB, val1, val2 );
        glNormal3f( (*vit).normal.getX(),
                    (*vit).normal.getY(),
                    (*vit).normal.getZ() );
        glVertex3f( (*vit).pos.getX(),
                    (*vit).pos.getY(),
                    (*vit).pos.getZ() );
    }
    glEnd();
}

void Model::drawModelWithLighting_nh_nv( const Point3 &eyepos, const Point3 &lpos )
{
    VertexList::const_iterator vit;

    glBegin( GL_TRIANGLES );
    for( vit = vlist.begin(); vit != vlist.end(); vit++ ) {
        Vector3 v = eyepos.subtract( (*vit).pos ).normalize();
        Vector3 l = lpos.subtract( (*vit).pos ).normalize();
        Vector3 h = v.add( l ).normalize();
        Vector3 n = (*vit).normal;

        double nh = n.dotProduct( h );
        double nv = n.dotProduct( v );
        double val1 = ( asin( (*vit).normal.getX() ) / M_PI ) + 0.5;
        double val2 = ( asin( (*vit).normal.getY() ) / M_PI ) + 0.5;

        assert( !isnan( nh ) && !isinf( nh ) );
        assert( !isnan( nv ) && !isinf( nv ) );

        nh = ( nh / 2.0 ) + 0.5;
        nv = ( nv / 2.0 ) + 0.5;
        //nv = 1.0 - nv;

        glTexCoord2f( nh, nv );
        glMultiTexCoord2fARB( GL_TEXTURE1_ARB, val1, val2 );
        glNormal3f( (*vit).normal.getX(),
                    (*vit).normal.getY(),
                    (*vit).normal.getZ() );
        glVertex3f( (*vit).pos.getX(),
                    (*vit).pos.getY(),
                    (*vit).pos.getZ() );
    }
    glEnd();
}

