
#ifndef MODEL_H_INCLUDED
#define MODEL_H_INCLUDED

#include <vertexlist.h>

class Model
{
public:
    Model( const VertexList &vertices );
    ~Model( void );

    void centreModel( void );

    /**
     * Sends all the triangles for the model, sends texture coordinates
     * for both texture units for spherical mapping.
     */
    void drawModel( void );

    /**
     * Sends all the triangles for the model, the primary texture unit
     * is indexed using v.h and n.h, and the secondary texture unit is
     * indexed using spherical mapping.
     */
    void drawModelWithLighting_vh_nh( const Point3 &v, const Point3 &lpos );

    /**
     * Sends all the triangles for the model, the primary texture unit
     * is indexed using n.l and n.v, and the secondary texture unit is
     * indexed using spherical mapping.
     */
    void drawModelWithLighting_nl_nv( const Point3 &v, const Point3 &lpos );

    /**
     * Sends all the triangles for the model, the primary texture unit
     * is indexed using n.h and n.v, and the secondary texture unit is
     * indexed using spherical mapping.
     */
    void drawModelWithLighting_nh_nv( const Point3 &v, const Point3 &lpos );

private:
    VertexList vlist;
};

#endif // MODEL_H_INCLUDED
