
#include <node.h>

Node::Node( void )
{
    forward = identityMatrix();
    reverse = identityMatrix();
}

Node::~Node( void )
{
}

PrimitiveList Node::foldModel( const Matrix4x4 &xform ) const
{
    PrimitiveList mylist;
    PrimitiveList::const_iterator pit;
    vector<BaseObject *>::const_iterator it;
    Matrix4x4 newx = multiplyMatrix( xform, forward ); // XXX: is the order right?

    for( it = objects.begin(); it != objects.end(); it++ ) {
        PrimitiveList cur = (*it)->foldModel( newx );

        for( pit = cur.begin(); pit != cur.end(); pit++ ) {
            mylist.push_back( (*pit) );
        }
    }

    return mylist;
}

VertexList Node::foldToVerticies( const Matrix4x4 &xform ) const
{
    VertexList mylist;
    vector<BaseObject *>::const_iterator it;
    Matrix4x4 newx = multiplyMatrix( xform, forward );

    for( it = objects.begin(); it != objects.end(); it++ ) {
        VertexList::const_iterator vit;
        VertexList cur = (*it)->foldToVerticies( newx );

        for( vit = cur.begin(); vit != cur.end(); vit++ ) {
            mylist.push_back( (*vit) );
        }
    }

    return mylist;
}

void Node::applyMatrix( const Matrix4x4 &xform )
{
    forward = multiplyMatrix( forward, xform );
    reverse = inverseMatrix( forward );
}

bool Node::closestIntersect( const Point3 &origin, const Vector3 &dir,
                             double *t, Vector3 *norm,
                             Material **mat ) const
{
    vector<BaseObject *>::const_iterator it;
    Point3 neworigin = multiplyPointByMatrix( reverse, origin );
    Vector3 newdir = multiplyVectorByMatrix( reverse, dir );
    Vector3 newnormal;

    for( it = objects.begin(); it != objects.end(); it++ ) {
        if( (*it)->closestIntersect( neworigin, newdir, t, &newnormal, mat ) ) {
            *norm = multiplyVectorByMatrix( forward, newnormal );
            return true;
        }
    }

    return false;
}

void Node::setMaterial( Material *m, bool override )
{
    vector<BaseObject *>::const_iterator it;
    for( it = objects.begin(); it != objects.end(); it++ ) {
        (*it)->setMaterial( m, override );
    }
}

void Node::setSampleRate( int newrate )
{
    vector<BaseObject *>::const_iterator it;
    for( it = objects.begin(); it != objects.end(); it++ ) {
        (*it)->setSampleRate( newrate );
    }
}

