
#ifndef PRIMITIVE_H_INCLUDED
#define PRIMITIVE_H_INCLUDED

#include <point3.h>
#include <vector3.h>
#include <boundingbox.h>
#include <baseobject.h>

class Primitive : public BaseObject
{
public:
    Primitive( void ) : mat( 0 ), rate( 1 ) {}
    virtual ~Primitive( void ) {}

    virtual BoundingBox getBoundingBox( void ) const = 0;
    virtual bool isPointLight( void ) const { return false; }

    /* Allow primitives to be lights. */
    virtual Point3 getRandomLightPosition( const Point3 &closest,
                                           double *probability ) const
    {
        *probability = 0.0;
        return Point3::origin;
    }

    /* Return intersect as a t value from the given direction and a normal. */
    virtual bool closestIntersect( const Point3 &origin, const Vector3 &dir,
                                   double *t, Vector3 *normal,
                                   Material **mat ) const = 0;

    void setMaterial( Material *m, bool override = true )
    {
        if( override || !mat ) mat = m;
    }

    void setSampleRate( int newrate ) { rate = newrate; }
    int getSampleRate( void ) const { return rate; }

    Material *getMaterial( void ) const { return mat; }

private:
    Material *mat;
    int rate;
};

#endif // PRIMITIVE_H_INCLUDED
