
#ifndef SCENEPARSER_H_INCLUDED
#define SCENEPARSER_H_INCLUDED

#include <stl.h>
#include <list>
#include <fstream>
#include <material.h>
#include <camera.h>
#include <node.h>
#include <baseobject.h>
#include <vertexlist.h>

class SceneReader;

class SceneParser
{
public:
    explicit SceneParser( const char *filename );
    ~SceneParser( void );

    enum LightingMode {
        None,
        Ashikhmin,
        Phong
    };

    Node *getNode( const char *name ) const;
    Camera getCamera( void ) const { return camera; }
    int getWidth( void ) const { return width; }
    int getHeight( void ) const { return height; }
    int getGridWidth( void ) const { return gridwidth; }
    bool superSample( void ) const { return supersample; }
    int superSampleDepth( void ) const { return superdepth; }
    LightingMode getLightingMode( void ) const { return lmode; }
    bool fatalError( void ) const { return fatal; }

private:
    SceneReader *reader;
    list<Material *> materials;
    list<Node *> nodes;
    Camera camera;
    int width, height, gridwidth;
    bool supersample;
    int superdepth;
    LightingMode lmode;
    bool fatal;

    void parseMaterial( void );
    void parseCamera( void );
    Node *parseNode( void );
    Material *getMaterial( const char *name ) const;
    void readPolygons( Node *n, VertexList &vlist );
};

#endif // SCENEPARSER_H_INCLUDED
