

#ifndef VECTOR3_H_INCLUDED
#define VECTOR3_H_INCLUDED

#include <math.h>
#include <assert.h>
#include <utils.h>

class Vector3
{
public:
    Vector3( void )
        : x( 0.0 ), y( 0.0 ), z( 0.0 ), normalized( false ) {}
    Vector3( const double &xpos, const double &ypos, const double &zpos )
        : x( xpos ), y( ypos ), z( zpos ), normalized( false ) {}
    ~Vector3( void ) {}

    Vector3 cross( const Vector3 &right ) const
    {
        return Vector3( (y * right.z) - (z * right.y), (z * right.x) - (x * right.z),
                       (x * right.y) - (y * right.x) );
    }

    Vector3 subtract( const Vector3 &right ) const
    {
        return Vector3( x - right.x, y - right.y, z - right.z );
    }

    Vector3 add( const Vector3 &right ) const
    {
        return Vector3( x + right.x, y + right.y, z + right.z );
    }

    Vector3 scalarMult( double s ) const
    {
        return Vector3( x * s, y * s, z * s );
    }

    double lengthSquared( void ) const
    {
        return (x * x) + (y * y) + (z * z);
    }

    double length( void ) const
    {
        return sqrt( lengthSquared() );
    }

    Vector3 normalize( void ) const
    {
        if( !normalized ) {
            double l = length();
            //assert( l != 0.0 );
            if( l == 0.0 ) {
                nx = 0.0;
                ny = 0.0;
                nz = 0.0;
            } else {
                nx = x / l;
                ny = y / l;
                nz = z / l;
            }
            normalized = true;
        }
        return Vector3( nx, ny, nz );
    }

    double dotProduct( const Vector3 &right ) const
    {
        return ( x * right.x ) + ( y * right.y ) + ( z * right.z );
    }

    Vector3 rotate( double angle, const Vector3 &axis ) const
    {
        // Got information from http://mathworld.wolfram.com/
        angle = -angle;
        double s = sin(angle);
        double c = cos(angle);
        Vector3 tmp1 = scalarMult( c );
        Vector3 tmp2 = tmp1.add( axis.scalarMult( axis.dotProduct( *this )
                                                  * ( 1.0 - c ) ) );
        Vector3 tmp3 = tmp2.add( cross( axis ).scalarMult( s ) );
        return tmp3;

/*
        return *this * c
             + axis * ((axis * *this) * (1.0f - c))
             + (*this % axis) * s;
*/
    }

    double getX( void ) const { return x; }
    double getY( void ) const { return y; }
    double getZ( void ) const { return z; }

private:
    double x, y, z;
    mutable bool normalized;
    mutable double nx, ny, nz;
};

#endif // VECTOR3_H_INCLUDED
